import logging
import os
from functools import lru_cache
from http.client import HTTPConnection, HTTPSConnection
from json import JSONEncoder, JSONDecoder
import socket
from urllib.parse import urlparse, ParseResult


logging.getLogger().setLevel(logging.DEBUG)
log = logging.getLogger(__name__)


def _target() -> ParseResult:
    target = os.environ.get('NGROK_URL')

    if target is None:
        raise Exception('Environment variables for NGROK_URL is not set')

    log.debug(f'Target proxy will be: [{target}]')

    return urlparse(target)


def handler(event, context):
    log.debug(f'Proxying request [{event}]')
    cxn = (HTTPSConnection if _target().scheme == 'https' else HTTPConnection)(_target().hostname, _target().port)
    try:
        log.debug('Requesting')
        cxn.request(
            'POST', '/lambda',
            JSONEncoder().encode(dict(event=event, context={}, env_vars={k: v for k, v in os.environ.items()
                                                                         if 'AWS_XRAY' not in k})),
            headers={'Content-type': 'application/json'})

        log.debug('Responsing')
        resp = cxn.getresponse()
        if resp.status != 200:
            log.error(f'Response failed: [{resp.status}]')
            raise Exception(resp.read().decode('utf8'))
        else:
            result = JSONDecoder().decode(resp.read().decode('utf8'))
            return result.get('data', None)
    except Exception as e:
        log.error(f'Operation failed: [{e}]')
        raise




import importlib
import logging
import os
from functools import lru_cache
from typing import Tuple, Callable

from flask import Flask, request

app = Flask(__name__)

logging.basicConfig(level=logging.DEBUG)

log = logging.getLogger(__name__)


def _module_and_handler() -> Tuple[str, str]:
    module = os.environ.get('SHEPHERD_MODULE', None)
    if module is None:
        raise Exception('Environment variable [SHEPHERD_MODULE] must be set')

    parts = module.split('.')
    return '.'.join(parts[:-1]), parts[-1]


@lru_cache
def _handler() -> Callable:
    module_name, handler_name = _module_and_handler()
    return getattr(importlib.import_module(module_name), handler_name)


def _config_env(env_vars: dict):
    for k, v in env_vars.items():
        if os.environ.get(k, None) is None:
            log.info(f'Setting environment variable: [{k}] = [{v}]')
            os.environ[k] = v


@app.post('/lambda')
def lambda_handler():
    log.debug('Received new request from lambda proxy')
    payload = request.json
    event, context, env_vars = payload['event'], payload['context'], payload['env_vars']
    _config_env(env_vars)

    try:
        _handler()(event, context)

        log.debug('Proxy request processing complete.')
        return dict(status=200, message='OK')
    except Exception as e:
        log.error('Lambda invocation failed')
        log.error(e)
        return dict(status=500, message=str(e))


if __name__ == '__main__':
    log.info('Starting local lambda listener')
    app.run(host='0.0.0.0', port=8888)




import os
from zipfile import ZipFile

import boto3
import argparse
import tempfile
from os import path
from pyngrok import ngrok
from sys import argv
from pathlib import Path
from rich import print


script_dir = path.dirname(path.realpath(__file__))

_verbose: bool = False


def _debug(msg):
    if _verbose:
        print(f'[bold green]DEBUG[/bold green] {msg}')


def _info(msg):
    print(f'[bold blue]INFO[/bold blue] {msg}')


def mount(args):
    client = boto3.client('lambda')
    waiter = client.get_waiter('function_updated')

    # create new lambda handler
    with tempfile.TemporaryDirectory() as tmpdir:
        lambda_path = path.join(tmpdir, 'lambda.zip')

        _debug("Creating lambda zip file")
        with ZipFile(lambda_path, 'w') as zip:
            lambda_content = Path(path.join(script_dir, '../lambda/index.py')).read_text()
            zip.writestr('index.py', lambda_content)

        lambda_zip_bytes = Path(lambda_path).read_bytes()

        _info('Updating function code')
        client.update_function_code(
            FunctionName=args.function_name,
            ZipFile=lambda_zip_bytes
        )

        _debug('Waiting for code to have been updated')
        waiter.wait(
            FunctionName=args.function_name
        )

        _info('Updating function configuration')
        client.update_function_configuration(
            FunctionName=args.function_name,
            Handler='index.handler'
        )


def start(args):
    client = boto3.client('lambda')
    waiter = client.get_waiter('function_updated')

    # start ngrok tunnel
    _debug('Connecting to ngrok')
    http_tunnel = ngrok.connect(addr=args.port)

    _info(f'Pointing the lambda at our tunnel [underline]{http_tunnel.public_url}[/underline]')
    existing_configuration = client.get_function_configuration(
        FunctionName=args.function_name
    )
    lambda_env = existing_configuration['Environment']['Variables']
    lambda_env['NGROK_URL'] = http_tunnel.public_url
    client.update_function_configuration(
        FunctionName=args.function_name,
        Environment={
            'Variables': lambda_env
        }
    )

    waiter.wait(
        FunctionName=args.function_name
    )

    _debug('Loading handler')
    os.environ['SHEPHERD_MODULE'] = args.python_handler

    from aws_shepherd.local import app

    _info('Loading lambda handler')
    app.run(host='0.0.0.0', port=args.port)


parser = argparse.ArgumentParser()
parser.add_argument('-v', '--verbose', default=False, help='Show debug logs', action='store_true')

command_parsers = parser.add_subparsers()

mount_parser = command_parsers.add_parser('mount')
mount_parser.set_defaults(func=mount)
mount_parser.add_argument('function_name')

start_parser = command_parsers.add_parser('start')
start_parser.set_defaults(func=start)
start_parser.add_argument('-p', '--port', default=8888)
start_parser.add_argument('function_name')
start_parser.add_argument('python_handler')


def main():
    args = parser.parse_args(argv[1:])

    global _verbose
    _verbose = args.verbose

    args.func(args)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['aws_shepherd',
 'aws_shepherd.cli',
 'aws_shepherd.lambda',
 'aws_shepherd.tester']

package_data = \
{'': ['*']}

install_requires = \
['Flask>=2.0.2,<3.0.0',
 'boto3>=1.18.61,<2.0.0',
 'pyngrok>=5.1.0,<6.0.0',
 'rich>=10.3.0,<11.0.0']

entry_points = \
{'console_scripts': ['shepherd = aws_shepherd.cli:main']}

setup_kwargs = {
    'name': 'aws-shepherd',
    'version': '0.1.0',
    'description': 'Sepherds AWS Lamb(das) to your local machine',
    'long_description': None,
    'author': 'Futurebank',
    'author_email': 'opensource@futurebank.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
